// AHThread.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001
*/
/* 
  This thread sends a query to AuctionHawk at
  http://www.auctionhawk.com/search.cgi, and pulls out the matching
  results. They are written to the display table as SearchResult objects.

  Sample AH query for "x-men 1":
    "http://www.auctionhawk.com/search?i=on&menu=search&query=x-men+1"
  
   Meaning of the fields:
      "i=on" only get auctions which contain an image
      "menu=search" means search by name
      "query=" is the actual query (comics_name+issue)
*/
/* CHANGES
      April 11th: altered AH URL from search.php to search
      August 24th: altered the AH URL to simplify the query part;
                   the format of the URL results had changed at AH
                   as well, but the RegExpr could still extract them
      August 27th: added fixAmper() method to manipulate the URLs 
                   returned by AH from eBay
*/

import java.io.*;
import java.net.*;
import java.util.*;
import COM.stevesoft.pat.*; // Regular Expression


public class AHThread extends Thread
{
  private String title, issue, enIDPass;
  private ComicSearch searchIntf;

  public AHThread(String Ctitle, String Cissue, String pass, ComicSearch cs)
  {
    title = Ctitle;
    issue = Cissue;
    enIDPass = pass;
	searchIntf = cs;
  }

  
  public void run()
  { 
    try { 
      //String Ahkey = "?i=on" + "&menu=search" + "&query=" + 
      //                 URLEncoder.encode(title) + "+" + issue;
      // URL u = new URL ( "http://www.auctionhawk.com/search.php" + Ahkey );

      String Ahkey = "?query=" + URLEncoder.encode(title) + "+" + issue;
      URL u = new URL ( "http://www.auctionhawk.com/search" + Ahkey );

	  URLConnection conn = u.openConnection();    // query Auction Hawk

      if (enIDPass != null)  // Send authorization though the proxy server
        conn.setRequestProperty("Proxy-Authorization", "Basic " + enIDPass);
      conn.setDoOutput( true );

      // read results
      BufferedReader dis = new BufferedReader ( 
                  new InputStreamReader ( conn.getInputStream() ) );

      String line;
	  SearchResults sr;
      while ( ((line=dis.readLine())!=null) && 
                !searchIntf.stopSearchPressed() ){
          sr = AucHawkFilter(line + "\n");    // filter a line at a time
          if (sr != null && sr.hasValues())
            searchIntf.writeRow(sr);      // write the results to the table
      }
    } 	
	catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }
	searchIntf.threadFinished("AH");
  } // end of run()


  SearchResults AucHawkFilter(String htmldata)
  // filter information from a line, and search for title/issue
  // if the title/issue is found then return a SearchResults object
  {
    String anchor = link(htmldata);         // pull out the link text
    String clntag = cleanMatch(anchor);     // remove some tags

    // Regular expression for comics title name and issue
    // (?) : ignore lower and upper case, | means OR
    // \\D : not a digit
    Regex comics = new Regex("((?i)" + title + 
                           ") (([#]{0,}|(?i)no. )" + issue + "\\D)");
    comics.search( clntag );    // search for title/issue in link text
    if (comics.didMatch())
	  return new SearchResults( pullURL(clntag), 
	                            pullAnchorText(clntag), "AH" );
    return null;
  } // end of AucHawkFilter()



  // Find matching result delimited by "<a class=auction ... >"
  // Ex. input : <a class=auction target=new href="/exec/item.php?x=l&url=
  //              http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?
  //              ViewItem&item=542560255">ASTONISHING X-MEN
  //              #1-4 Magneto Rogue Sabre...</a>
  // output :
  //      target=new href="/exec/item.php?x=l&url=
  //       http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?
  //      ViewItem&item=542560255">
  //       ASTONISHING X-MEN #1-4 Magneto Rogue Sabre...
  String link(String s)
  {
    StringBuffer sb = new StringBuffer("");
    Regex http = new Regex("<a class=auction");
    Regex end = new Regex("</a>");

    http.search("" + s);           // so not a null string
    end.search("" + http.right()); // Initial search
    sb.append( http.stringMatched() + end.left() + end.stringMatched() );
    return sb.toString();
  } // end of link()


    String pullURL(String s)
    // Find a URL in "s" beginning with "http:://"
    // e.g. http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?ViewItem&item=542560255
    {
      Regex tag = new Regex("(?i)http://");
      Regex end1 = new Regex("\">|\')\">|\',\'\')\">|\"\">"); // may have javaScript

      tag.search("" + s);    // so not a null string
      end1.search( tag.stringMatched() + tag.right() );
      if (end1.didMatch())
        return fixAmper( end1.left() );
      return "missing url";
    }


    String fixAmper( String s)
    // AuctionHawk converts the '&' before 'item=' into '%26' in eBay URLs
    // Convert it back so that eBay understands it
    {
       int amPos;
       if ( (s.indexOf("ebay.com") != -1) &&
            (amPos = s.indexOf("%26item=")) != -1) {
         String newS = s.substring(0,amPos) + "&" + s.substring(amPos+3);
         // System.out.println("NEW: " + newS);
         return newS;
       }
       return s;
    }

    String pullAnchorText (String s)
    // The anchor text starts after the ">", and ends with the
    // "<" of </a>
    {
      Regex begin = new Regex(">");
      Regex end = new Regex("(?i)</a>");

      begin.search( s );
      end.search( begin.right() );
      return end.left();
    }


	String cleanMatch(String s)
	// Remove the tags <b>, </b>, <br>
	{
	  s = cutTag(s, "<b>");
	  s = cutTag(s, "</b>");
	  s = cutTag(s, "<br>");
	  return s;
	}


    // Replace the tags in s with ""'s
    // Ex.input :cutTag("<br>x-men 1<br>", "<br>");
    // output : "x-men 1"
    String cutTag(String s, String tag)
	{
      Regex line = new Regex(tag, "");
      return line.replaceAll("" + s);
    }


} // end of AHThread class
