// GGThread.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001

   Changes:
      21st Nov. 2001: Modified URL connection to send
      a sourceid name/value pair in GET, and the "User-Agent"
      request header. This makes Google think the query is
      coming from an Opera browser.
*/
/* 
  This thread sends a query to Google at
  http://www.google.com/search, and pulls out the matching
  results. They are written to the display table as SearchResult objects.

  Sample query for "x-men 1":
    "http://www.google.com/search?as_q=%22x-men+1%22&num=50
             &btnG=Google+Search&as_occt=any&as_dt=i&safe=off"

  Meaning of the fields:
     "as_q" = : the query (comics_title+issue)
     "num=50" : max number of results
	 "btnG" : the search button 'pressed'
     "as_occt=any", "as_dt=i" and "safe=off" : required (probably)

*/

import java.io.*;
import java.net.*;
import java.util.*;
import COM.stevesoft.pat.*; // Regular Expression


public class GGThread extends Thread
{
  private String title, issue, enIDPass;
  private ComicSearch searchIntf;

  public GGThread(String Ctitle, String Cissue, String pass, ComicSearch cs)
  {
    title = Ctitle;
    issue = Cissue;
    enIDPass = pass;
	searchIntf = cs;
  }


  public void run()
  {
    try {
      /* String GGkey = "?as_q=%22" + URLEncoder.encode(title) + "+" +
                     issue + "%22" +
                     "&num=50&btnG=Google+Search&as_occt=any&as_dt=i&safe=off"; */
      // modify query to include sourceID name/value pair
      String GGkey = "?q=%22" + URLEncoder.encode(title) + "+" +
                     issue + "%22" +
					 "&sourceid=opera&num=50";

      URL u = new URL ( "http://www.google.com/search" + GGkey );
      URLConnection conn = u.openConnection();   // send query to Google
 
      // include "User-Agent" header
      conn.setRequestProperty("User-Agent", "Mozilla/5.0 (Windows 95; U) Opera 5.12 _ [en]");

      if (enIDPass != null)    // Send authorization though the proxy server
        conn.setRequestProperty("Proxy-Authorization", "Basic " + enIDPass);
      conn.setDoOutput( true );

      BufferedReader dis = new BufferedReader ( 
	          new InputStreamReader ( conn.getInputStream() ) );
      String line;
	  SearchResults sr;
      Regex schPoint = new Regex("(?i)</A>");   // for delimiting the text searches

      while ( ((line=dis.readLine())!=null) && 
              !searchIntf.stopSearchPressed() ) {
        schPoint.search(line);
        if (schPoint.didMatch()) {    // filter text once a </a> tag is found
          sr = GGFilter(line);
          if (sr != null && sr.hasValues())
            searchIntf.writeRow(sr);
        }
      }
    } 
	catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }
    searchIntf.threadFinished("GG");
  } // end of run()


  SearchResults GGFilter (String htmldata)
  // Filter information from the text, and search for the title/issue.
  // If the title/issue is found then return a SearchResults object.
  { 
    // Filter result data for only "<a href=http:// ... > ... </a>"
    String resultlink = findLink(htmldata);    // pull out the link text
    String clntag = cleanMatch(resultlink);    // remove some tags

    // Regular expression for comics title name and issue
    // (?) : ignore lower and upper case,  | means OR
    // + : Search this expression backwards
    Regex comics = new Regex("((?i)" + title + 
                        "(| | [A-Za-z]+ | #)" + issue + ")+");

    comics.search( clntag );
    if (comics.didMatch()) {     // found title & issue
      String URLText = pullURL(clntag);
      String anchorText = pullAnchorText(clntag);

	  Regex missurl = new Regex ("<table");     // filter for missing URL
      missurl.search(URLText);
      if ( missurl.didMatch() )
	    return null;
      if (!themeTitle(anchorText))    // no themes, thank you
 	    return new SearchResults( URLText, anchorText, "GG");
    }
    return null;
  }


  String findLink (String dLink)
  // extract a link. e.g. "<a href=http:// ... > ... </a>"
  { 
    Regex startLnk = new Regex("<(?i)(a href)=(?i)(http)://");
    Regex endLnk = new Regex("</(?i)(a)>");
    startLnk.search(dLink);
    if (startLnk.didMatch()) {
      endLnk.search(startLnk.stringMatched() + startLnk.right());
      if (endLnk.didMatch())
        return endLnk.left() + endLnk.stringMatched();
    } 
    return "";
  }


  String cleanMatch(String s)
  // remove several tags
  {
    s = cutTag(s, "<b>");  s = cutTag(s, "<B>");
    s = cutTag(s, "</b>"); s = cutTag(s, "</B>");
    s = cutTag(s, "<br>"); s = cutTag(s, "</BR>");
    s = cutTag(s, "<p>");  s = cutTag(s, "<P>");
    s = cutTag(s, "</p>"); s = cutTag(s, "</P>");
    return s;
  }


  String cutTag(String s, String tag)
  // Replace the tags in s with ""'s
  {
    Regex line = new Regex(tag, "");
    return line.replaceAll("" + s);   // not null string
  }


  // Extract the text between "<a href=" and ">"
  // ex. input: <a href=http://www.marvel.com/xmen.html>
  // output: www.marvel.com/xmen.html
  String pullURL(String s)
  {
    Regex tag = new Regex("(?i)<a href=");
    Regex end = new Regex(">");
    tag.search( "" + s );  // not null string
    end.search( tag.right() );
    if (end.didMatch())
      return end.left();
    return "missing url";
  }


  // Extract the text after the ">" to </a>
  // ex.input : <a href=http://www.marvel.com/xmen.html>x-men issue 1</a>
  // output : x-men issue 1
  String pullAnchorText (String s)
  {
    Regex begin = new Regex(">");
    Regex end = new Regex("</");
    begin.search( s );
    end.search( begin.right() );
    return end.left();;
  }


  private boolean themeTitle(String anchorText)
  // return true if the title contains "themes"
  // Used to filter out some common bad hits
  {
     Regex r = new Regex("(?i)themes");
     r.search(anchorText);
     return r.didMatch();
  }

} // end of GGThread class
