// ResultTModel.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001
*/

// This file contains two classes: ResultTModel and TModelColumnData.

/*
  This JTable has 4 columns
     1. The number of the row (integer)
     2. Web title
     3. URL of the Web title (as a string)
     4. Source ID for the Search engine/site where the information comes from
 
   The data for each row is stored in a SearchResults object.

*/

import java.util.Vector;
import javax.swing.JLabel;
import javax.swing.table.AbstractTableModel;


class ResultTModel extends AbstractTableModel 
{
  public static final int COL_NO = 0;     // row number
  public static final int COL_WEB = 1;    // anchor text/title
  public static final int COL_URL = 2;    // URL string
  public static final int COL_SOURCE = 3; // source of result

  private Vector vec;

  public static final TModelColumnData columns[] = {
    new TModelColumnData( "No.", 2, JLabel.CENTER ),
    new TModelColumnData( "Web Title", 250, JLabel.LEFT ),
    new TModelColumnData( "URL", 350, JLabel.LEFT ),
    new TModelColumnData( "Source", 3, JLabel.CENTER )
  };

  
  public ResultTModel ()
  { vec = new Vector(); }

  public int getRowCount() 
  {  return vec == null ? 0 : vec.size();  }

  public int getColumnCount() 
  { return columns.length;  } 

  public String getColumnName(int column) 
  {  return columns[column].title;  }
 

  public Object getValueAt(int row, int col) 
  {
    if (row < 0 || row >= getRowCount()) 
	  return "";

    SearchResults sr = (SearchResults) vec.elementAt(row);
    switch (col) {
      case COL_NO: return new Integer(sr.getRowPosn()); 
      case COL_WEB: return sr.getAnchorText();
      case COL_URL: return sr.getUrlText();
      case COL_SOURCE: return sr.getResultSource();
    }
    return "";
  }
  
  // insert data into a table row
  public void insert(int row, SearchResults sr)
  {
    if (row < 0)
      row = 0;
    else if (row > vec.size())
      row = vec.size();

    sr.setRowPosn(row+1);    // so row numbering begins at 1
    vec.insertElementAt(sr, row);
  }

  public void clear()
  {  vec.clear(); }


} // end of ResultTModel.java



// *******************************************************************************

class TModelColumnData
{
  public String title;
  public int width;
  public int alignment;

  public TModelColumnData(String t, int w, int align) 
  {
    title = t;
    width = w;
    alignment = align;
  }
}

