
// SilverThread.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001

*/
/* 
  This thread sends a query to Nick Simon's Silver Age Covers
  search engine at http://www.cs.ait.ac.th/cgi-bin/csearcher/,
  and pulls out the matching results. They are written to the 
  display table as SearchResult objects.

  Sample query for "x-men 1":
     http://www.cs.ait.ac.th/cgi-bin/csearcher/?p0=x-men&issueno=1&
                  all=no&period=none&clientmode=java

   Meaning of fields:
     "p0" holds the title
     "issueno" holds the issue
     "all=no" means search all titles
     "period=none" means search the entire time range of the db
     "clientmode=java" means return the result in 'java' format

  The search engine is available at two sites: AIT and on fivedots at PSU.
  The AIT version is the 'standard' while the one at PSU is 'experimental'.
*/


import java.io.*;
import java.net.*;
import java.util.*;
import COM.stevesoft.pat.*; // Regular Expression


public class SilverThread extends Thread
{
  private String title, issue, enIDPass;
  private ComicSearch searchIntf;

  public SilverThread(String Ctitle, String Cissue, String pass, ComicSearch cs)
  {
    title = Ctitle;
    issue = Cissue;
    enIDPass = pass;
    searchIntf = cs;
  }


  public void run()
  {
    // contact Silver Search with a query, and create input stream for results
    BufferedReader dis = getWebInfo(); 

    try {
      String line, tempdata = "";
	  SearchResults sr;

      while ( ((line=dis.readLine())!=null) && 
              !searchIntf.stopSearchPressed() ) {
        // Store result lines until "" is encountered
        // "" delimits each result
        tempdata += line;
        if (line.equals("")) {
          sr = SilFilter(tempdata);
          if (sr != null && sr.hasValues())
            searchIntf.writeRow(sr);    // store results in the table
          tempdata = "";    // reset tempdata
        }
      }
    } 	
	catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }
    searchIntf.threadFinished("SS");
  } // End of run()


  private BufferedReader getWebInfo()
  // contact Silver Search with a query, and create input stream for results
  {
    BufferedReader dis = null;

    try {
      String Silkey = "p0=" + URLEncoder.encode(title) +
                      "&all=no&period=none&clientmode=java" +
                      "&issueno=" + issue + "&submit=Submit+Search";
      // URL u = new URL ( "http://fivedots.coe.psu.ac.th/cgi-bin/ad/csearcher/" + Silkey);
	  URL u = new URL( "http://www.cs.ait.ac.th/cgi-bin/csearcher/" + Silkey);
      URLConnection conn = u.openConnection();

      if (enIDPass != null)    // Send authorization though the proxy server
        conn.setRequestProperty("Proxy-Authorization", "Basic " + enIDPass);
      conn.setDoOutput( true );

      // send query using the POST method
      PrintWriter out = new PrintWriter(conn.getOutputStream());
      out.println( Silkey );
      out.close();

      // read results
      dis = new BufferedReader ( 
	            new InputStreamReader ( conn.getInputStream() ) );
    } 	
	catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }
    return dis;
  }


  public SearchResults SilFilter (String htmldata)
  // Filter information from the text, and search for the title/issue.
  // If the title/issue is found then return a SearchResults object.
  {
    String clntag = cleanMatch(htmldata);   // remove some tags

    // Regular expression for comics title name and issue
    // (?) : ignore lower and upper case,  | means OR
    // [A-Zz-z] : match any char,   + : match at least 1 char
    Regex comics = new Regex("(?i)" + title + "(| | [A-Za-z]+)" + issue);
    comics.search( clntag );
    if (comics.didMatch())
	  return new SearchResults( pullURL(clntag), comics.stringMatched(), "SS" );
    return null;
  }


  String cleanMatch(String s)
  // Remove the tags <b>, </b>, <br>
  {
    s = cutTag(s, "<b>");
    s = cutTag(s, "</b>");
    s = cutTag(s, "<br>");
    return s;
  }


  String cutTag(String s, String tag)
  // Delete all occurrences of Tag in s
  {
    Regex line = new Regex(tag, "");
    return line.replaceAll("" + s);  // so not null string
  }


  // Extract the text beginning with "http://"
  // and ending with ".htm" or ".html"
  String pullURL(String s)
  {
    Regex tag = new Regex("(?i)http://");
    Regex end = new Regex(".(?i)htm[l]{0,1}");

    tag.search("" + s);  // so not null string
    end.search( tag.stringMatched() + tag.right() );

    if (end.didMatch())
      return end.left() + end.stringMatched();
    return "missing url";
  } // end of pullURL()

} // end SilverThread class
